package de.jottyfan.csv2camt.xml;

import java.io.FileWriter;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.jottyfan.csv2camt.csv.CsvBean;
import de.jottyfan.csv2camt.csv.CsvLine;

/**
 * 
 * @author jotty
 * 
 */
public class CamtWriter {

	protected Document generateXml(CsvBean bean) throws Exception {
		if (bean.getIban() == null || bean.getIban().length() < 10) {
			throw new Exception(String.format("invalid iban: %s", bean.getIban()));
		} else if (bean.getKontostandFinal() == null) {
			throw new Exception("no kontostandFinal");
		}
		String receiver = "Evangelische-Freikirchliche Gemeinde Dresden Süd-Ost";
		StringBuilder buf = new StringBuilder("KtoNr");
		buf.append(bean.getIban().substring(bean.getIban().length() - 10));
		buf.append("_");
		buf.append(bean.getKontostandDatum().replace(".", "-"));
		buf.append("_0000"); // no creation time in csv
		String MsgIdText = buf.toString();
		buf = new StringBuilder();
		buf.append(bean.getKontostandDatum().replace(".", "-"));
		buf.append("T00:00:00.000+02:00"); // no creation time in csv
		String kontostandzeitpunkt = buf.toString();
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document xml = builder.newDocument();
		xml.setXmlStandalone(true);

		Element document = xml.createElement("Document");
		document.setAttribute("xmlns", "urn:iso:std:iso:20022:tech:xsd:camt.052.001.06");
		xml.appendChild(document);

		Element BkToCstmrAcctRpt = new XmlTag(xml, "BkToCstmrAcctRpt", document).get();
		Element GrpHdr = new XmlTag(xml, "GrpHdr", BkToCstmrAcctRpt).get();
		new XmlTag(xml, "MsgId", GrpHdr).withText(MsgIdText);
		Element CreDtTm = new XmlTag(xml, "CreDtTm", GrpHdr).withText(kontostandzeitpunkt).get();
		new XmlTag(xml, "Nm", new XmlTag(xml, "MsgRcpt", GrpHdr).get()).withText(receiver);
		Element Rpt = new XmlTag(xml, "Rpt", BkToCstmrAcctRpt).get();
		new XmlTag(xml, "Id", Rpt).withText(MsgIdText);
		Rpt.appendChild(CreDtTm.cloneNode(true));
		new XmlTag(xml, "IBAN", new XmlTag(xml, "Id", new XmlTag(xml, "Acct", Rpt).get()).get()).withText(bean.getIban());
		Element Bal = new XmlTag(xml, "Bal", Rpt).get();
		new XmlTag(xml, "Cd", new XmlTag(xml, "CdOrPrtry", new XmlTag(xml, "Tp", Bal).get()).get()).withText("CLAV");
		new XmlTag(xml, "Amt", Bal).withAttr("Ccy", bean.getWaehrung())
				.withText(bean.getKontostandFinal().replace(".", "").replace(",", ".")); // not sure if correct...
    new XmlTag(xml, "CdtDbtInd", Bal).withText("CRDT");
		new XmlTag(xml, "DtTm", new XmlTag(xml, "Dt", Bal).get()).withText(CreDtTm.getTextContent());
		
		for (CsvLine trans : bean.getList()) {
			Element Ntry = new XmlTag(xml, "Ntry", Rpt).get();
			new XmlTag(xml, "Amt", Ntry).withAttr("Ccy", trans.Waehrung())
					.withText(trans.Betrag().replace(".", "").replace(",", "."));
			new XmlTag(xml, "CdtDbtInd", Ntry).withText("CRDT"); // not sure if this is right
			new XmlTag(xml, "Sts", Ntry).withText("BOOK"); // not sure if this is right
			new XmlTag(xml, "Dt", new XmlTag(xml, "BookgDt", Ntry).get()).withText(XmlTag.yyyyMMdd(trans.Buchungstag()));
			new XmlTag(xml, "Dt", new XmlTag(xml, "ValDt", Ntry).get()).withText(XmlTag.yyyyMMdd(trans.Wert()));
			Element BkTxCd = new XmlTag(xml, "BkTxCd", Ntry).get();
			Element Domn = new XmlTag(xml, "Domn", BkTxCd).get();
			new XmlTag(xml, "Cd", Domn).withText("PMNT"); // not sure if this is right
			Element Fmly = new XmlTag(xml, "Fmly", Domn).get();
			new XmlTag(xml, "Cd", Fmly).withText("RCDT"); // not sure if this is right
			new XmlTag(xml, "SubFmlyCd", Fmly).withText("ESCT"); // not sure if this is right
			Element Prtry = new XmlTag(xml, "Prtry", BkTxCd).get();
			new XmlTag(xml, "Cd", Prtry).withText("NTRF+166"); // not sure if this is right
			new XmlTag(xml, "Issr", Prtry).withText("DK"); // not sure if this is right
			Element TxDtls = new XmlTag(xml, "TxDtls", new XmlTag(xml, "NtryDtls", Ntry).get()).get();
			new XmlTag(xml, "Amt", TxDtls).withAttr("Ccy", trans.Waehrung()).withText(trans.Betrag());
			new XmlTag(xml, "CdtDbtInd", TxDtls).withText("CRDT"); // not sure if this is right
			Element BkTxCd_Ntry = new XmlTag(xml, "BkTxCd", TxDtls).get();
			Element Domn_Ntry = new XmlTag(xml, "Domn", BkTxCd_Ntry).get();
			new XmlTag(xml, "Cd", Domn_Ntry).withText("PMNT"); // not sure if this is right
			Element Fmly_Ntry = new XmlTag(xml, "Fmly", Domn_Ntry).get();
			new XmlTag(xml, "Cd", Fmly_Ntry).withText("RCDT"); // not sure if this is right
			new XmlTag(xml, "SubFmlyCd", Fmly_Ntry).withText("ESCT"); // not sure if this is right
			Element Prtry_Ntry = new XmlTag(xml, "Prtry", BkTxCd_Ntry).get();
			new XmlTag(xml, "Cd", Prtry_Ntry).withText("NTRF+166"); // not sure if this is right
			new XmlTag(xml, "Issr", Prtry_Ntry).withText("DK"); // not sure if this is right
			Element RltdPties = new XmlTag(xml, "RltdPties", TxDtls).get();
			new XmlTag(xml, "Nm", new XmlTag(xml, "Dbtr", RltdPties).get()).withText(trans.Beguenstigter_Auftraggeber());
			new XmlTag(xml, "Nm", new XmlTag(xml, "Cdtr", RltdPties).get()).withText(receiver);
			Element RmtInf = new XmlTag(xml, "RmtInf", TxDtls).get();
			new XmlTag(xml, "Ustrd", RmtInf).withText(String.format("Referenz %s", trans.Kundenreferenz()));
			new XmlTag(xml, "Ustrd", RmtInf).withText(trans.Verwendungszweck());
		}
		return xml;
	}

	public void write(CsvBean bean, String filename) throws Exception {
		Document document = generateXml(bean);
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer transformer = tf.newTransformer();
		StringWriter writer = new StringWriter();
		transformer.transform(new DOMSource(document), new StreamResult(writer));

		String xmlString = writer.toString();

		try (FileWriter fileWriter = new FileWriter(filename)) {
			fileWriter.write(xmlString);
			fileWriter.flush();
		}
	}
}
